(ns cipher-analytical-machine.cli
  (:require
   [cipher-analytical-machine.caesar :as caesar]
   [clojure.string :as cs]
   [clojure.tools.cli :refer [parse-opts]])
  (:gen-class))

(def cipher-options
  #{"Caesar"})

(def cli-options
  [["-m" "--message MESSAGE" "The message will be encrypted or decrypted."]
   ["-k" "--key KEY" "The key will be used to encrypt or decrypt."]
   ["-c" "--cipher CIPHER" "The cipher will be used to encrypt or decrypt a message."
    :default "Caesar"]
   ["-e" "--encrypt" "Encrypt the message."]
   ["-d" "--decrypt" "Decrypt the message."]
   ["-h" "--help"]])

(defn usage [options-summary]
  (->> ["This is cipher-analytical-machine. It can help you to learn how ciphers works."
        ""
        "Usage: cipher-analytical-machine [options]"
        ""
        "Options:"
        options-summary
        ""]
       (cs/join \newline)))

(defn error-msg [errors]
  (str "The following errors occurred while parsing your command:\n\n"
       (cs/join \newline errors)))

(defn map-has-keys?
  "Return true if all keys in the map."
  [map keys]
  (every? #(contains? map %) keys))

(defn validate-args
  "Validate command line arguments. Either return a map indicating the program
  should exit (with an error message, and optional ok status), or a map
  indicating the action the program should take and the options provided."
  [args]
  (let [{:keys [options arguments errors summary]} (parse-opts args cli-options)]
    (cond
      (:help options)
      {:exit-message (usage summary) :ok? true}

      errors
      {:exit-message (error-msg errors)}

      (map-has-keys? options [:encrypt, :decrypt])
      {:exit-message (error-msg ["You can't use enctypt and decrypt mode at the same time!!!"])}

      (and (map-has-keys? options [:cipher, :key])
           (contains? options :message)
           (or (contains? options :encrypt)
               (contains? options :decrypt)))
      {:options options :arguments arguments}

      :else
      {:exit-message (usage summary)})))

(defn exit
  [exit-message status]
  (println exit-message)
  (System/exit (if status 0 1)))

(defn actions
  [options arguments]
  (println (str options arguments))
  (let [message (:message options)
        key (Integer/parseInt (:key options))
        symbols "abc"]
    (println
     (cond
       (contains? options :encrypt)
       (caesar/encrypt-message message key symbols)

       (contains? options :decrypt)
       (caesar/decrypt-message message key symbols)))))

